/*
 *  asteroids.h
 *  Asteroids
 *
 *  
 *
 */

#if defined(__APPLE__) || defined(MACOSX)
	#include <GLUT/glut.h>
//	#include <Carbon/Carbon.h>
#else
	#include <GL/glut.h>
#endif

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#define PI 3.141592654

#define FRAMEWIDTH 640
#define FRAMEHEIGHT 480

#define NUMBERSTARS 512
#define MAX_NUMBER_PLAYERS 2

//essentially the gamespeed
#define FRAMES_PER_SECOND 30

//how long a ghost of a shield or thrust will appear
#define FADEOUTS 625
//how long the ship is invincible at the beginning of a level
#define INVINCIBLE_LENGTH 3000

//at what amount of surplus shields will the system overload?
//i.e., 0.75 means overload after 6 extra on maxShields of 8
#define OVERLOAD_FACTOR 0.5


//print timestamps on shots
//#define DEBUG_SHOTS

//print crystal information
//#define DEBUG_CRYSTALS

//eternal invincibility; for debugging, of course
//#define ETERNAL_INVINCIBILITY

//are random numbers actually sorta random?
#define RANDOM_ON true


#define true 1
#define false 0

typedef unsigned boolean;

/****************************************************
 *
 * Data Structures
 *
 **************************/

typedef struct coordinate {
	float x;
	float y;
} Coordinate, Vector;

//a hit!
typedef struct hitData {
	short hitCode; //used by functions to pass special data about the hit
	int mass; //mass of the object that hit caused the hit
	Vector direction; //where the hit came from
} HitData;

//Asteroid Sizes; index of array containing radius
#define ASTEROID_BIG 2
#define ASTEROID_MEDIUM 1
#define ASTEROID_SMALL 0
//int asteroidSizes[3] = {10, 20, 30};

typedef struct asteroid {
	int size; //actually the index of an array which defines the radii of an asteroid
	GLubyte color[3]; //the color; probably will never change
	HitData impacted; //has this asteroid been hit yet? If so, it gets tossed.
} AsteroidStruct;

typedef struct spaceship {
	int playerNumber;
	int shipSize; //the radius of the ship
	GLubyte color[3]; //RGB values range from 0 to 255 using GLubyte
	float rotationAngle; //in *degrees*
	
	int invinciblityTimeToLive; //how long until the ship will become vulnerable again
	int drawShieldsTimeToLive; //shields drawn for a little bit after their use; this is their expiration date
	int strength; //how strong is our ship?
	boolean thrusting; //is our intrepid craft supposed to be accelerating?
	boolean decelerating; //is our intrepid craft supposed to be accelerating?
	int drawThrustUntil; //When the thrust fire will fade out
	short rotating; //-1 if rotating counterclockwise, 0 if not rotating, 1 if rotating clockwise
	float rotationScaleFactor; //the percentage of the total rotation to move by per frame (see rotateShip)
} SpaceshipStruct;

typedef struct shot { //a bullet
	int owner; //who fired this bullet?
	int timeToLive; //when this bullet was shot, from GLUT_ELAPSED_TIME
	GLubyte color[3];
	boolean impacted; //has this bullet hit anything yet? If so, it gets tossed.
} ShotStruct;

//kerblooie Sizes; number of particles to create in an explosion
#define KERBLOOIE_HUGE 3
#define KERBLOOIE_BIG 2
#define KERBLOOIE_MEDIUM 1
#define KERBLOOIE_SMALL 0
//int kerblooieSizes[4] = {24, 36, 48, 60}; //defined in asteroids.c

typedef struct kerblooie { //a particle from an explosion
	int lifespan; //how long this particle's life will actually be
	int timeToLive; //milliseconds of life this particle has left
	float size; //diameter of the particle
	GLubyte color[4]; //original color and fade percentage, interpreted as alpha value
} KerblooieStruct;


//Crystal Specimen Codes (can be array subscripts)
#define CRYSTAL_SHIELDPOWER 0

#define CRYSTAL_NUMBER_OF_SPECIMENS 1

#define CRYSTAL_NORMAL 0
#define CRYSTAL_IMPACTED 1
#define CRYSTAL_RETRIEVED 2
typedef struct crystal { //a six-armed crystal for powerups, etc.
	int timeToLive; //milliseconds of life this particle has
	float rotationAngle; //the current rotation angle of the crystal around <0,1,1>
	float rotationIncrement; //how much to increment the rotation each frame
	float radius; //radius of crystal
	float thickness; //radius of crystal's arms
	short crystalFlavor; //what kind of crystal this is
	GLubyte color[4]; //original color and fade percentage, interpreted as alpha value
	HitData status; //filled with CRYSTAL_STATUS
} CrystalStruct;


typedef struct object { //superclass for all objects
	Coordinate position; //current centerpoint of the object
	Vector direction; //the direction vector of *movement*
	int mass; //how heavy this object is; used in collisions
	
	AsteroidStruct asteroid;
	SpaceshipStruct *spaceship;
	ShotStruct shot;
	KerblooieStruct kerblooie;
	CrystalStruct *crystal;
} Asteroid, Spaceship, Shot, Kerblooie, Crystal, Object;

//node for a linked list
struct node {
	struct node *previousNode;
	Object object;
	struct node *nextNode;
};
typedef struct node Node;
typedef Node *NodePtr;

/****************************************************
 *
 * Macros (General Purpose)
 *
 **************************/

//Index of player number in array
#define P(x) ( (x)-1 )

//distance formula between two Coordinates
#define distanceCoordinate(coordPtr1, coordPtr2) ( 									\
	sqrt( pow(coordPtr2.x - coordPtr1.x,2) + pow(coordPtr2.y - coordPtr1.y,2) )		\
)

#define magnitudeVector(vector) ( sqrt( pow( (vector).x,2) + pow( (vector).y,2) ) )


/****************************************************
 *
 * Function Prototypes
 *
 **************************/
extern void gameInit(int);
extern void display(void);
extern void drawStarfield();
extern void drawStatisticsHeader();
extern void drawSpaceShip(Spaceship *);
extern void drawShots(NodePtr);
extern void drawAsteroids(NodePtr);
extern void drawKerblooies(NodePtr);
extern void drawCrystals(NodePtr);
extern void drawCrystalArm(float, float);
extern void drawHalfSphere(int, int, int);
extern void moveSpaceShip(Spaceship *, float);
extern void moveShots(NodePtr *, float);
extern void moveAsteroids(NodePtr *, float);
extern void moveKerblooies(NodePtr *, float);
extern void adjustCoordinate(Coordinate *,Vector *,float); //might return coord pointer?
extern void collideObjects(Object *, Object *);
extern void idle(void);
extern void fireShot(Spaceship *);
extern HitData isCrystalHit(Crystal *, NodePtr *);
extern HitData isAsteroidHit(Object *, NodePtr *);
extern HitData isShipHit(Spaceship *, NodePtr *, NodePtr *, NodePtr *);
extern void accelerateShip(Spaceship *, float);
extern void decelerateShip(Spaceship *, float);
extern void rotateShip(Spaceship *, float);
extern void impactShip(Spaceship *, HitData);
extern void resurrectShip(int);
extern void spawnStarfield();
extern Spaceship spawnSpaceship(int);
extern Object spawnAsteroid(int,int,int,Vector);
extern void spawnKerblooie(int, int, GLubyte [],Coordinate, Vector, NodePtr *);
extern void spawnCrystal(int, Coordinate, NodePtr *);
extern void startLevel(int);
extern void endGame(int);
extern void handleKeyboard(unsigned char, int, int);
extern void handleSpecialKeyboard(int, int, int);
extern void handleKey(int);
extern void handleKeyboardUp(unsigned char, int, int);
extern void handleSpecialKeyboardUp(int, int, int);
extern void handleKeyUp(int);

//linked-list prototypes
extern NodePtr createList();
extern void insertNode( NodePtr *, Object);
extern Object removeNode( NodePtr *);
extern boolean isEmpty( NodePtr );

